#!/usr/bin/env python

import os, sys, string, re

#----------------------------------------------------------------------
# Url class   [ Steve Dekorte, Summer 1999 ]
#
# Used for parsing and composing urls.
#
# Parse example:
#
#  p = Url.Url()
#  p.url_("http://www.egroups.com/info/UserGroupsPage?search=hello")
#  searchValue = p.argumentsDict().get('search', '') # searchValue  = "hello"
#  searchValue = p.removeArgument_('method') # searchValue  = "hello"
#
# Compose example:
#
#    p = Url.Url().domain_('www.egroups.com')
#    p.addPath_('info')
#    p.addPath_('UserGroupsPage')
#    p.setArgument_toValue_('search', 'hello')
#    url = p.unparsedUrl() # url = "http://www.egroups.com/info/UserGroupsPage?search=hello"
#
#----------------------------------------------------------------------

def escapeReplace(matchobj):
  return string.upper("%%%02x" % ord(matchobj.group(0)))

class Url:
  _escape = re.compile(
    r"(?x)"
    r"([^a-zA-Z0-9_.-])"
  )

  def __init__(self):
    self._url = ""
    self._domainPrefix = ""
    self._domain = ""
    self._path = ""
    self._argumentsDict = {}
    self._endsPathWithSlash = 0

  #--- Getters/Setters ---

  def url_(self, o): self._url = o; self.parse(); return self
  def url(self): return self._url

  def unparsedUrl(self): self.unparse(); return self.url()

  def protocol_(self, o): self._protocol = o; return self
  def protocol(self): return self._protocol

  def domain_(self, o): self._domain = o; return self
  def domain(self): return self._domain

  def fullDomain(self): 
    if not self.domain(): return ""
    return "http://" + self.domain()

  def path_(self, o): self._path = o; return self
  def path(self): return self._path

  def addPath_(self, path):
    if not self.path() and path: self.path_("/")
    newPath = os.path.join(self.path(), path)
    self.path_(newPath)
    return self

  def splitPath(self):
    pair = os.path.split(self.path())
    self.path_(pair[0])
    return self

  def argumentsDict_(self, o): self._argumentsDict = o; return self
  def argumentsDict(self): return self._argumentsDict
  
  def removeArgument_(self, arg):
    if self.argumentsDict().get(arg, None):
      del self.argumentsDict()[arg]
    return self

  def set_to_(self, arg, value):
    return self.setArgument_toValue_(arg, value)

  def setArgument_toValue_(self, arg, value):
    self.argumentsDict()[arg] = value
    return self

  def endsPathWithSlash_(self, o): self._endsPathWithSlash = o; return self
  def endsPathWithSlash(self): return self._endsPathWithSlash

  #--- Actions ---

  def unparse(self):
    self._url = self.fullDomain()
    if self.path():
      self._url = self._url + self.path()
      if self.endsPathWithSlash():
        self._url = self._url + '/'
    argString = self.argumentsString()
    if argString:
      self._url = self._url + "?" + argString
    return self

  def argumentsString(self):
    aString = ""
    dict = self.argumentsDict()
    for key in dict.keys():
      value = dict[key]
      if value != "" and not (value is None):
        if len(aString): aString = aString + "&"
        aString = aString + key + "=" + self.realescape(str(value)) 
    return aString

  def parse(self):
    #log("url = " + self.url())
    self.parseDomain()
    self.parsePath()
    self.parseArguments()
    #log("unparsedUrl: " + self.unparsedUrl())
    #log("---")
    return self

  def parseDomain(self):
    aString = self.url()
    domain = ""
    if aString[:len("http://")] == "http://":
      aString = aString[len("http://"):]
      parts = string.split(aString, '/')
      domain = parts[0]
    self.domain_(domain)
    #log("domain = " + self.domain())
    return self

  def parsePath(self):
    url = self.url()
    if self.domain():
      url = url[len(self.fullDomain()):]
    index = string.rfind(url, '?') 
    if index != -1:
      url = url[:index]
    self.path_(url)
    #log("path = " + self.path())
    return self

  def parseArguments(self):
    dict = {}
    url = self.url()[len(self.fullDomain() + self.path()):]
    if url and url[0] == "?":
      url = url[1:]
    if url:
      parts = string.split(url, '&')
      for part in parts:
        peices = string.split(part, '=')
        if not len(peices): break
        key = peices[0]
        value = peices[1]
        dict[key] = value
    self.argumentsDict_(dict)
    #log("args = " + repr(self.argumentsDict()))
    return self

  def __str__(self):
    return self.unparsedUrl()


  def realescape(self, s): # URL-encode data
    if not s: return None
    return self._escape.sub(escapeReplace, s)

def test():
  # testing url parsing
  p = Url()
  p.url_("http://www.egroups.com")
  p.url_("http://www.egroups.com/")
  p.url_("http://www.egroups.com/UserGroupsPage?")
  p.url_("http://www.egroups.com/info/general/UserGroupsPage")
  p.url_("http://www.egroups.com/info/general/UserGroupsPage?")
  p.url_("http://www.egroups.com/info/UserGroupsPage?search=hello&start=1")
  p.url_("http://www.egroups.com/info/UserGroupsPage/?search=hello&start=1")
  p.url_("/info/UserGroupsPage?search=hello&start=1")
  p.url_("UserGroupsPage?search=hello&start=1")
  p.removeArgument_("start")
  #log("p = " + p.unparse().url())

#test()
